from math import sqrt

def jacobi(phi, rho, h, tol=0.001):
    error = 10000.0  
    niter = 0
    while error > tol:
        niter += 1
        error = 0.0
        phi_old = phi[:]
        for n in range(1, len(phi)-1):
            phi[n] = (phi_old[n+1] + phi_old[n-1] + h**2 * rho[n] ) / 2.0
            error += (phi[n] - phi_old[n] )**2
        error = sqrt(error)
        if niter > 5000:
            print "Failed to converge"
            break
    return niter

# Computational parameters
a = 0
b = 1.0
h = 0.02

N = int( (b-a) / h)
x = [0.0]
for n in range(1, N):
    x.append(x[n-1] + h)

# source and analytic solution
rho = []
phi_correct = []
for x_i in x:
    rho.append(2 * x_i**2 - 1)
    phi_correct.append(-x_i/3.0 + x_i**2 / 2.0 - x_i**4 / 6.0)

# Numerical solution
phi = [0.0] * N

niter1 = jacobi(phi, rho, h, 1e-4)
print "Jacobi converged in {0} iterations".format(niter1)
# Error to analytic solution
error = 0.0
for n in range(len(phi)):
    error += (phi[n] - phi_correct[n] )**2
error = sqrt(error)

print "Error to analytic solution", error


